@@ -53,8 +53,8 @@ CDelegate::~CDelegate()
 void CDelegate::paint(QPainter* pPainter, const QStyleOptionViewItem& option, const QModelIndex& index) const
 {
     QString strText = index.data(Qt::DisplayRole).toString();
-    qint64 sizeParent = index.parent().data(MaxSizeRole).toLongLong();
-    qint64 sizeTotal = index.data(SizeRole).toLongLong();
+    quint64 sizeParent = index.parent().data(MaxSizeRole).toULongLong();
+    quint64 sizeTotal = index.data(SizeRole).toULongLong();
 
     if (option.state & QStyle::State_Selected)
     {
@@ -75,7 +75,7 @@ void CDelegate::paint(QPainter* pPainter, const QStyleOptionViewItem& option, co
                     ColorMap::iterator iter = mapSizes.begin();
                     for ( ; iter != mapSizes.end(); iter++)
                     {
-                        qint64 size = iter.value();
+                        quint64 size = iter.value();
 
                         //qDebug() << iter.key() << size << sizeParent << index.data(Qt::DisplayRole).toString();
 
