@@ -51,6 +51,9 @@ CScanWindow::CScanWindow(QWidget* pParent)
 #ifdef Q_WS_MAEMO_5
     setAttribute(Qt::WA_Maemo5StackedWindow);
 #endif
+
+    QSettings settings;
+    m_Limit = settings.value("limit", DEFAULT_MAX_LEVEL).toInt();
 }
 
 //-----------------------------------------------------------------------------
@@ -59,6 +62,14 @@ CScanWindow::CScanWindow(QWidget* pParent)
 */
 CScanWindow::~CScanWindow()
 {
+    QSettings settings;
+    settings.setValue("limit", m_Limit);
+
+    if (m_pModel)
+    {
+        m_pModel->clear();
+    }
+
     if (m_pScanThread)
     {
         m_pScanThread->cancel();
@@ -101,7 +112,7 @@ void CScanWindow::initialize(QString strFolder, bool bScanApps)
     m_pTreeWidget->setModel(m_pSortModel);
 
     // The usage grid
-    m_pUsageGrid = new CUsageGrid(pSplitter, m_pTreeWidget);
+    m_pUsageGrid = new CUsageGrid(pSplitter, m_pTreeWidget, m_Limit);
     pSplitter->addWidget(m_pUsageGrid);
 
     QVBoxLayout* pLayout = new QVBoxLayout();
@@ -164,6 +175,8 @@ void CScanWindow::initialize(QString strFolder, bool bScanApps)
     Q_ASSERT(bOk);
     bOk = connect(m_pTreeWidget, SIGNAL(doubleClicked(const QModelIndex&)), this, SLOT(onItemDoubleClicked(const QModelIndex&)));
     Q_ASSERT(bOk);
+    bOk = connect(m_pTreeWidget, SIGNAL(expanded(const QModelIndex&)), this, SLOT(onItemExpanded(const QModelIndex&)));
+    Q_ASSERT(bOk);
 }
 
 //-----------------------------------------------------------------------------
@@ -193,11 +206,11 @@ void CScanWindow::startScan()
     m_pScanThread = new CScanThread(this, m_strFolder);
 
     bool bOk = false;
-    bOk = connect(m_pScanThread, SIGNAL(folderFound(const QString&)), this, SLOT(onFolderFound(const QString&)));
+    bOk = connect(m_pScanThread, SIGNAL(folderFound(const QString&, int)), this, SLOT(onFolderFound(const QString&, int)));
     Q_ASSERT(bOk);
     bOk = connect(m_pScanThread, SIGNAL(folderScanned()), this, SLOT(onFolderScanned()));
     Q_ASSERT(bOk);
-    bOk = connect(m_pScanThread, SIGNAL(fileFound(const QString&, const QString&, qint64)), this, SLOT(onFileFound(const QString&, const QString&, qint64)));
+    bOk = connect(m_pScanThread, SIGNAL(fileFound(const QString&, const QString&, quint64, int)), this, SLOT(onFileFound(const QString&, const QString&, quint64, int)));
     Q_ASSERT(bOk);
     bOk = connect(m_pScanThread, SIGNAL(progressMaximum(int)), this, SLOT(onProgressMaximum(int)));
     Q_ASSERT(bOk);
@@ -273,14 +286,23 @@ void CScanWindow::onTextChanged(const QString& strText)
 ** the tree widget.
 **
 ** \param strFolder The full path to the folder.
+** \param level The current hierarchy level.
 */
-void CScanWindow::onFolderFound(const QString& strFolder)
+void CScanWindow::onFolderFound(const QString& strFolder, int level)
 {
     if (m_pModel)
     {
         QStandardItem* pItem = new QStandardItem();
-        pItem->setData(TYPE_FOLDER, TypeRole);
+        if (level < m_Limit)
+        {
+            pItem->setData(TYPE_FOLDER, TypeRole);
+        }
+        else
+        {
+            pItem->setData(TYPE_UNSCANNED_FOLDER, TypeRole);
+        }
         pItem->setData(strFolder, PathRole);
+        pItem->setData(0, SizeRole);
 
         if (m_pCurrentItem)
         {
@@ -307,23 +329,14 @@ void CScanWindow::onFolderScanned()
 {
     if (m_pCurrentItem)
     {
-        // Calculate the total for the folder
-        qint64 total = 0;
-        qint64 maxSize = 0;
-        ColorMap mapSizes;
-        for (int i = 0; i < m_pCurrentItem->rowCount(); i++)
+        // Add the subfolders to the total (files are already added)
+        quint64 size = m_pCurrentItem->data(SizeRole).toULongLong(NULL);
+        QString strFilePath = m_pCurrentItem->data(PathRole).toString();
+
+        if (m_pCurrentItem->data(TypeRole).toInt() == TYPE_UNSCANNED_FOLDER && m_pCurrentItem->rowCount() == 0)
         {
-            qint64 size = m_pCurrentItem->child(i)->data(SizeRole).toLongLong();
-            Colors color = CMainWindow::getColorForPath(m_pCurrentItem->child(i)->data(PathRole).toString());
-            total += size;
-            mapSizes[color] += size;
-            maxSize = qMax(maxSize, size);
+            m_pCurrentItem->appendRow(new QStandardItem());  // Temp item to show the decoration
         }
-        QVariant vntSizes;
-        vntSizes.setValue(mapSizes);
-        m_pCurrentItem->setData(vntSizes, SizeMapRole);
-        m_pCurrentItem->setData(total, SizeRole);
-        m_pCurrentItem->setData(maxSize, MaxSizeRole);
 
         m_pCurrentItem = m_pCurrentItem->parent();
         if (m_pCurrentItem == NULL && m_pUsageGrid && m_pDelegate)
@@ -331,6 +344,10 @@ void CScanWindow::onFolderScanned()
             // This was the last item -> Create the usage grid
             m_pUsageGrid->initialize(m_pModel->item(0, 0), !m_bScanApps);
         }
+        else
+        {
+            addData(strFilePath, size);
+        }
     }
 }
 
@@ -341,17 +358,55 @@ void CScanWindow::onFolderScanned()
 ** \param strFilename The name of the file.
 ** \param strFilePath The file path.
 ** \param size The size of the file.
+** \param level The current hierarchy level.
 */
-void CScanWindow::onFileFound(const QString& strFilename, const QString& strFilePath, qint64 size)
+void CScanWindow::onFileFound(const QString& strFilename, const QString& strFilePath, quint64 size, int level)
 {
     if (m_pCurrentItem)
     {
-        QStandardItem* pItem = new QStandardItem();
-        pItem->setData(TYPE_FILE, TypeRole);
-        pItem->setData(strFilePath, PathRole);
-        pItem->setData(strFilename, Qt::DisplayRole);
-        pItem->setData(size, SizeRole);
-        m_pCurrentItem->appendRow(pItem);
+        addData(strFilePath, size);
+
+        if (level <= m_Limit)
+        {
+            QStandardItem* pItem = new QStandardItem();
+            pItem->setData(TYPE_FILE, TypeRole);
+            pItem->setData(strFilePath, PathRole);
+            pItem->setData(strFilename, Qt::DisplayRole);
+            pItem->setData(size, SizeRole);
+            m_pCurrentItem->appendRow(pItem);
+        }
+    }
+}
+
+//-----------------------------------------------------------------------------
+/**
+** Adds the size and color to the current item.
+**
+** \param strFilePath The file path.
+** \param size The size of the file.
+*/
+void CScanWindow::addData(const QString& strFilePath, quint64 size)
+{
+    if (m_pCurrentItem)
+    {
+        ColorMap mapSizes;
+        quint64 total = m_pCurrentItem->data(SizeRole).toULongLong(NULL);
+        quint64 maxSize = m_pCurrentItem->data(MaxSizeRole).toULongLong(NULL);
+        QVariant vntSizes = m_pCurrentItem->data(SizeMapRole);
+        if (vntSizes.canConvert<ColorMap>())
+        {
+            mapSizes = vntSizes.value<ColorMap>();
+        }
+
+        Colors color = CMainWindow::getColorForPath(strFilePath);
+        total += size;
+        mapSizes[color] += size;
+        maxSize = qMax(maxSize, size);
+
+        vntSizes.setValue(mapSizes);
+        m_pCurrentItem->setData(vntSizes, SizeMapRole);
+        m_pCurrentItem->setData(total, SizeRole);
+        m_pCurrentItem->setData(maxSize, MaxSizeRole);
     }
 }
 
@@ -418,6 +473,49 @@ void CScanWindow::onItemSelectionChanged(const QModelIndex& current, const QMode
     }
 }
 
+//-----------------------------------------------------------------------------
+/**
+** Slot which gets called when the an item is expanded. Scans the files for
+** the unscanned branches.
+**
+** \param index The expanded item.
+*/
+void CScanWindow::onItemExpanded(const QModelIndex& index)
+{
+    if (index.data(TypeRole).toInt() == TYPE_UNSCANNED_FOLDER)
+    {
+        QStandardItem* pParent = m_pModel->itemFromIndex(m_pSortModel->mapToSource(index));
+
+        if (pParent)
+        {
+            // Remove the temp item
+            if (pParent->rowCount() == 1 && pParent->child(0, 0)->data(TypeRole).isNull())
+            {
+                pParent->removeRow(0);
+            }
+
+            QDir dir(index.data(PathRole).toString());
+            dir.setFilter(QDir::Files | QDir::Hidden | QDir::NoSymLinks);   // We already have the folders
+
+            QFileInfoList list = dir.entryInfoList();
+            for (int i = 0; i < list.size(); ++i)
+            {
+                QFileInfo fileInfo = list.at(i);
+
+                QStandardItem* pItem = new QStandardItem();
+                pItem->setData(TYPE_FILE, TypeRole);
+                pItem->setData(fileInfo.absoluteFilePath(), PathRole);
+                pItem->setData(fileInfo.fileName(), Qt::DisplayRole);
+                pItem->setData(fileInfo.size(), SizeRole);
+
+                pParent->appendRow(pItem);
+            }
+
+            pParent->setData(TYPE_FOLDER, TypeRole);       // This is now scanned
+        }
+    }
+}
+
 //-----------------------------------------------------------------------------
 /**
 ** Slot which gets called when the an item is double clicked. Changes the
