@@ -24,11 +24,12 @@
 **
 ** \param pParent The parent for the widget.
 */
-CUsageGrid::CUsageGrid(QWidget* pParent, QTreeView* pTreeView) : QWidget(pParent)
+CUsageGrid::CUsageGrid(QWidget* pParent, QTreeView* pTreeView, int limit) : QWidget(pParent)
 {
     m_pTreeView = pTreeView;
     m_pRootItem = NULL;
     m_pRecursive = false;
+    m_Limit = limit;
 
     clear();
 }
@@ -171,17 +172,17 @@ void CUsageGrid::drawGrid(QPainter& painter, QStandardItem* pItem, const QRectF&
     {
         pItem->setData(rect, AreaRole);
 
-        qint64 total = pItem->data(SizeRole).toLongLong();
+        quint64 total = pItem->data(SizeRole).toULongLong();
         if (total > 0)
         {
             // Calculate the total size for each row so we know their relative sizes
             int rows = (int)sqrt(pItem->rowCount());
-            QList<qint64> rowTotals;
+            QList<quint64> rowTotals;
             for (int i = 0; i < pItem->rowCount(); i++)
             {
                 if (pItem->child(i))
                 {
-                    qint64 size = pItem->child(i)->data(SizeRole).toLongLong();
+                    quint64 size = pItem->child(i)->data(SizeRole).toULongLong();
                     if (i % rows == 0)
                     {
                         rowTotals.append(size);
@@ -193,8 +194,8 @@ void CUsageGrid::drawGrid(QPainter& painter, QStandardItem* pItem, const QRectF&
                 }
             }
 
-            qint64 newRowTotal = 0;
-            qint64 newColumnTotal = 0;
+            quint64 newRowTotal = 0;
+            quint64 newColumnTotal = 0;
 
             int row = -1;
             qreal x = rect.left();
@@ -202,7 +203,7 @@ void CUsageGrid::drawGrid(QPainter& painter, QStandardItem* pItem, const QRectF&
             qreal h = rect.height();
             qreal w = rect.width();
 
-            if (pItem->rowCount() == 0 || (rect.width() < 2 && rect.height() < 2) || (!bRecursive && pItem->parent()) || level > 3)
+            if (pItem->rowCount() == 0 || (rect.width() < 2 && rect.height() < 2) || (!bRecursive && pItem->parent()) || level > m_Limit)
             {
                 QString strName = pItem->data(Qt::DisplayRole).toString();
 
@@ -227,7 +228,7 @@ void CUsageGrid::drawGrid(QPainter& painter, QStandardItem* pItem, const QRectF&
             {
                 for (int i = 0; i < pItem->rowCount(); i++)
                 {
-                    qint64 size = pItem->child(i)->data(SizeRole).toLongLong();
+                    quint64 size = pItem->child(i)->data(SizeRole).toULongLong();
 
                     if (i % rows == 0)
                     {
@@ -239,7 +240,7 @@ void CUsageGrid::drawGrid(QPainter& painter, QStandardItem* pItem, const QRectF&
                         newRowTotal += rowTotals[row];
                         newColumnTotal = 0;
                     }
-                    qint64 rowTotal = rowTotals[row] == 0 ? 1 : rowTotals[row];
+                    quint64 rowTotal = rowTotals[row] == 0 ? 1 : rowTotals[row];
                     w = rect.width() * size / rowTotal;
                     x = rect.left() + rect.width() * newColumnTotal / rowTotal;
 
