@@ -25,7 +25,7 @@ class CUsageGrid : public QWidget
     Q_OBJECT
 
 public:
-    CUsageGrid(QWidget* pParent, QTreeView* pTreeView);
+    CUsageGrid(QWidget* pParent, QTreeView* pTreeView, int limit);
     ~CUsageGrid();
 
     void clear();
@@ -52,6 +52,7 @@ private:
     QPixmap m_pxGrid;               ///< The grid image
     QRect m_rectSelection;          ///< The selected item's rect
     bool m_pRecursive;              ///< True when the leaf nodes are shown. False when just the root is shown.
+    int m_Limit;                    ///< The limit for the hierarchy
 };
 
 #endif // USAGEGRID_H
